/*
 * Sistemas de Tempo Real, 2009/2010
 * Paulo Pedreiras, Luis Almeida
 * Sept 2009
 *
 * STR Linux/RTAI lab demo
 * 	4 RT tasks toggle a bit on the paralell port.
 * 	Bit status is sent to rt sharem memory
 * 	Linux user-space task reads rt shared memory and displays task status
 *
 * 	RT module
 *
 * Changelog:
 *  2006/09/21, Paulo Pedreiras
 *     	Ported from RTAI 1.0 to RTAI 3.3
 *     	Major changes: thread arg in init changed from int to long
 *     	task_preempt_always() not supported anymore
 *     	Makefile updated due to change in linux kernel from 2.4 to 2.6
 *     	macro to define init/cleanup functions (kernel 2.4->2.6)
 *
 *  2009/09/09, Paulo Pedreiras
 * 	ncurses based user-space task scope
 */

/*
 *  Common definitions
 */
#define NUM_TASKS 	4      		/* Number of RT tasks */
#define FIFO_ID 	0 			/* FIFO ID */
#define FIFO_SIZE   8000        /* FIFO size */

typedef struct {                /* Structure use to communicate events (instant, task, event type) */
  long long evt_time;
  unsigned char task_id;
  unsigned char task_act;
} Task_Evt;

/*
 * RTAI definititions
 */
#define USE_FPU 1        /* Uses FPU flag */
#define LPT_ADDR 0x378   /* Address of data register of paralell port */

#define PERIOD 1000000   /* Tasks base period in ns (RTAI) */
#define STACK_SIZE 3000  /* Tasks's stack size (RTAI) */

/* Task characterization: period in ms*/
#define TASK0_PERIOD 1000
#define TASK1_PERIOD 2000
#define TASK2_PERIOD 3000
#define TASK3_PERIOD 4000

/* rt_busy_sleep arg in ns */
#define MS2NS 1000000L
#define TASK0_LOAD  (100L*MS2NS)
#define TASK1_LOAD  (250L*MS2NS)
#define TASK2_LOAD  (500L*MS2NS)
#define TASK3_LOAD (1000L*MS2NS)


/*
 *  User-space scope definitions
 */
/* Display organization */
#define FIRST_COL 5		/* First column where task graphs are plotted */
#define FIRST_ROW 4 	/* First row where task graphs are plotted */
#define STEP_ROW  4		/* Rows between adjacent graphs */
#define RIGHT_MARGIN 4  /* Margin the right of the window */
#define MARK_RUNNING '-'
#define MARK_IDLE '_'
#define MARK_GORUNN '/'
#define MARK_GOIDLE '\\'

/* General configs */
#define DISP_STEP_MS 	100		/* Display step in ms */
#define TASK_TRIG 		0 		/* Task to be used as trigger [0..MAX_TASKS-1]*/
