/*
 * Sistemas de Tempo Real, 2009/2010
 * Paulo Pedreiras, Luis Almeida
 * Sept 2009
 *
 * STR Linux/RTAI lab demo
 * 	4 RT tasks toggle a bit on the paralell port.
 * 	Bit status is sent to rt sharem memory
 * 	Linux user-space task reads rt shared memory and displays task status
 *
 * 	RT module
 *
 * Changelog:
 *  2006/09/21, Paulo Pedreiras
 *     	Ported from RTAI 1.0 to RTAI 3.3
 *     	Major changes: thread arg in init changed from int to long
 *     	task_preempt_always() not supported anymore
 *     	Makefile updated due to change in linux kernel from 2.4 to 2.6
 *     	macro to define init/cleanup functions (kernel 2.4->2.6)
 *
 *  2009/09/09, Paulo Pedreiras
 * 		ncurses based user-space task scope
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <asm/io.h>

#include <rtai.h>
#include <rtai_sched.h>
#include <rtai_fifos.h>
#include <rtai_shm.h>
#include <rtai_nam2num.h>


/* Application configuration */
#include "app_defs.h"

MODULE_AUTHOR("Paulo Pedreiras, Luis Almeida <{pbrp,lda}@ua.pt>");
MODULE_DESCRIPTION("Runs 4 rt tasks periodically and toggles paralell port bits.");
MODULE_LICENSE("GPL");

#define ONE_SHOT        /* Selects Single Shot timer mode*/

/* RT task descriptors */
static RT_TASK utask0;
static RT_TASK utask1;
static RT_TASK utask2;
static RT_TASK utask3;

static unsigned char pps=0; /* Paralel port bit image */

/************************/
/* User Real-Time tasks */
/************************/

static void fun0(long t)
{
  Task_Evt tevt;

  /*
   * Task initializations
   */
  //printk("\n First instance task %d",t);

  /*
   * Task body
   */
  while(1) {

    /* Signal task start of execution */
    tevt.evt_time=rt_get_time_ns();     		/* get activation time and fill event structure */
    tevt.task_id=t;
    tevt.task_act='s';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld on  (%lld)",t,tevt.evt_time);     /* Debug message*/
    pps |= (0x1 << t); 								/* Set paralel port pin */
    outb(~pps,LPT_ADDR);

    /* Task's workload*/
    rt_busy_sleep(TASK0_LOAD);

    /*Signal task end of execution */
    tevt.evt_time=rt_get_time_ns();     /* get finish */
    pps &= (~(0x1 << t));       		/* Reset paralel port pin */
    outb(~pps,LPT_ADDR);
    tevt.task_id=t;      				/* Fill in event info and send to FIFO */
    tevt.task_act='f';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld finish (%lld)",t,rt_get_time_ns());	/* Task finish debug message*/

    /* Sleep until next activation */
    rt_task_wait_period();
  }
}


static void fun1(long t)
{
  Task_Evt tevt;

  /*
   * Task initializations
   */
  //printk("\n First instance task %d",t);

  /*
   * Task body
   */
  while(1) {

    /* Signal task start of execution */
    tevt.evt_time=rt_get_time_ns();     			/* get activation time and fill event structure */
    tevt.task_id=t;
    tevt.task_act='s';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld on  (%lld)",t,tevt.evt_time);    /* Debug message*/
    pps |= (0x1 << t); 								/* Set paralel port pin */
    outb(~pps,LPT_ADDR);


    /* Task's workload*/
    rt_busy_sleep(TASK1_LOAD);


    /*Signal task end of execution */
    tevt.evt_time=rt_get_time_ns();     /* get finish */
    pps &= (~(0x1 << t));       		/* Reset paralel port pin */
    outb(~pps,LPT_ADDR);
    tevt.task_id=t;      				/* Fill in event info and send to FIFO */
    tevt.task_act='f';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld finish (%lld)",t,rt_get_time_ns());	/* Task finish debug message*/

    /* Sleep until next activation */
    rt_task_wait_period();
  }
}


static void fun2(long t)
{
  Task_Evt tevt;

  /*
   * Task initializations
   */
  //printk("\n First instance task %d",t);

  /*
   * Task body
   */
  while(1) {



    /* Signal task start of execution */
    tevt.evt_time=rt_get_time_ns();     			/* get activation time and fill event structure */
    tevt.task_id=t;
    tevt.task_act='s';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld on  (%lld)",t,tevt.evt_time);    /* Debug message*/
    pps |= (0x1 << t); 								/* Set paralel port pin */
    outb(~pps,LPT_ADDR);


    /* Task's workload*/
    rt_busy_sleep(TASK2_LOAD);


    /*Signal task end of execution */
    tevt.evt_time=rt_get_time_ns();     	/* get finish */
    pps &= (~(0x1 << t));       			/* Reset paralel port pin */
    outb(~pps,LPT_ADDR);
    tevt.task_id=t;      					/* Fill in event info and send to FIFO */
    tevt.task_act='f';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld finish (%lld)",t,rt_get_time_ns());	/* Task finish debug message*/


    /* Sleep until next activation */
    rt_task_wait_period();
  }
}

static void fun3(long t)
{
  Task_Evt tevt;

  /*
   * Task initializations
   */
  //printk("\n First instance task %d",t);

  /*
   * Task body
   */
  while(1) {

    /* Signal task start of execution */
    tevt.evt_time=rt_get_time_ns();     			/* get activation time and fill event structure */
    tevt.task_id=t;
    tevt.task_act='s';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld on  (%lld)",t,tevt.evt_time);    /* Debug message*/
    pps |= (0x1 << t); 								/* Set paralel port pin */
    outb(~pps,LPT_ADDR);


    /* Task's workload*/
    rt_busy_sleep(TASK3_LOAD);


    /*Signal task end of execution */
    tevt.evt_time=rt_get_time_ns();     /* get finish */
    pps &= (~(0x1 << t));       		/* Reset paralel port pin */
    outb(~pps,LPT_ADDR);
    tevt.task_id=t;      				/* Fill in event info and send to FIFO */
    tevt.task_act='f';
    rtf_put(FIFO_ID,&tevt,sizeof(tevt));
    printk("\n %ld finish (%lld)",t,rt_get_time_ns());	/* Task finish debug message*/

    /* Sleep until next activation */
    rt_task_wait_period();
  }
}



/***************************************************/
/* Module Initialization and termination functions */
/***************************************************/
int __str_test_init(void)
{
  RTIME tick_period;
  RTIME startTime;
  int res;

  printk("[str_test] Initializing RTAI module ...\n");

  /* RT FIFO initialization */
  res =rtf_create(FIFO_ID, FIFO_SIZE);
  if (res) {
    printk("[str_test] RT Fifo creation error (%d)\n",res);
    return res;
  }

  /* Cotrol the use of FPU by linux foreground processes (kernel and its processes)*/
  rt_linux_use_fpu(USE_FPU);

  /* Initialize user RT tasks
   * args: RT_TASK structure, thread code address, thread  arg (long), stack size,
   * priority (lower numbers -> higher prio.), FPU use flag, context switch handler
   */
  rt_task_init(&utask0, fun0, 0, STACK_SIZE, 0, USE_FPU, 0);
  rt_task_init(&utask1, fun1, 1, STACK_SIZE, 1, USE_FPU, 0);
  rt_task_init(&utask2, fun2, 2, STACK_SIZE, 2, USE_FPU, 0);
  rt_task_init(&utask3, fun3, 3, STACK_SIZE, 3, USE_FPU, 0);

#ifdef ONE_SHOT
  //rt_set_oneshot_mode();
  printk("[str_test] One shot mode selected \n\r");
#else
  printk("[str_test] Periodic timer mode selected \n\r");
#endif

  /* Compute system tick (tick_period in internal timer units) */
  tick_period = start_rt_timer(nano2count(PERIOD));

  startTime = rt_get_time() + NUM_TASKS*tick_period*1000;

  /* Start User RT Tasks */
  /* args: RT_TASK structure, start time (absolute, in internal ticks), period (in internal ticks) */
  rt_task_make_periodic(&utask0, startTime, TASK0_PERIOD*tick_period);
  rt_task_make_periodic(&utask1, startTime, TASK1_PERIOD*tick_period);
  rt_task_make_periodic(&utask2, startTime, TASK2_PERIOD*tick_period);
  rt_task_make_periodic(&utask3, startTime, TASK3_PERIOD*tick_period);

  printk("[str_test] Module successfuly loaded!\n");
  return 0;
}


void __str_test_exit(void)
{
  int res;

  stop_rt_timer();
  rt_busy_sleep(10000000);

  /* Remove user RT tasks */
  rt_task_delete(&utask0);
  rt_task_delete(&utask1);
  rt_task_delete(&utask2);
  rt_task_delete(&utask3);

  /* Release FIFO  */
  res=rtf_destroy(FIFO_ID);
  if(res<0)
    printk("[str_test] Failed to destroy FIFO (%d)\n",res);

  printk("\n Finito!\n");
}

module_init(__str_test_init);
module_exit(__str_test_exit);
